@@ -23,6 +23,8 @@
 ///     and could be removed.
 /// 3) a `RESERVED_FOR_TABLE_ALIAS` array with keywords reserved in a
 /// "table alias" context.
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
 
 /// Defines a string constant for a single keyword: `kw_def!(SELECT);`
 /// expands to `pub const SELECT = "SELECT";`
@@ -41,7 +43,8 @@ macro_rules! define_keywords {
     ($(
         $ident:ident $(= $string_keyword:expr)?
     ),*) => {
-        #[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Ord)]
+        #[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Ord, Hash)]
+        #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
         #[allow(non_camel_case_types)]
         pub enum Keyword {
             NoKeyword,
@@ -69,6 +72,7 @@ define_keywords!(
     ALL,
     ALLOCATE,
     ALTER,
+    ANALYZE,
     AND,
     ANY,
     APPLY,
@@ -84,6 +88,8 @@ define_keywords!(
     AT,
     ATOMIC,
     AUTHORIZATION,
+    AUTOINCREMENT,
+    AUTO_INCREMENT,
     AVG,
     AVRO,
     BEGIN,
@@ -185,6 +191,7 @@ define_keywords!(
     EXECUTE,
     EXISTS,
     EXP,
+    EXPLAIN,
     EXTENDED,
     EXTERNAL,
     EXTRACT,
@@ -343,6 +350,7 @@ define_keywords!(
     RELEASE,
     RENAME,
     REPEATABLE,
+    REPLACE,
     RESTRICT,
     RESULT,
     RETURN,
@@ -437,6 +445,7 @@ define_keywords!(
     VARYING,
     VAR_POP,
     VAR_SAMP,
+    VERBOSE,
     VERSIONING,
     VIEW,
     VIRTUAL,
@@ -459,6 +468,8 @@ define_keywords!(
 pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     // Reserved as both a table and a column alias:
     Keyword::WITH,
+    Keyword::EXPLAIN,
+    Keyword::ANALYZE,
     Keyword::SELECT,
     Keyword::WHERE,
     Keyword::GROUP,
@@ -490,6 +501,8 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
 pub const RESERVED_FOR_COLUMN_ALIAS: &[Keyword] = &[
     // Reserved as both a table and a column alias:
     Keyword::WITH,
+    Keyword::EXPLAIN,
+    Keyword::ANALYZE,
     Keyword::SELECT,
     Keyword::WHERE,
     Keyword::GROUP,
